==> newrelic-agent-4.9.0-sources.jar: com/newrelic/agent/servlet/ServletUtils.java <==
package com.newrelic.agent.servlet;

import java.util.Arrays;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.logging.Level;

import com.newrelic.agent.Agent;
import com.newrelic.agent.Transaction;
import com.newrelic.agent.attributes.AttributeNames;
import com.newrelic.agent.service.ServiceFactory;
import com.newrelic.agent.util.StackTraces;
import com.newrelic.api.agent.Request;

public class ServletUtils {

    private static String SERVLET_EXCEPTION_CLASS_NAME = "javax.servlet.ServletException";

    public static final String NEWRELIC_IGNORE_ATTRIBUTE_NAME = "com.newrelic.agent.IGNORE";
    public static final String NEWRELIC_IGNORE_APDEX_ATTRIBUTE_NAME = "com.newrelic.agent.IGNORE_APDEX";
    public static final String APPLICATION_NAME_PARAM = "com.newrelic.agent.APPLICATION_NAME";

    public static final String TRANSACTION_NAME_PARAM = "com.newrelic.agent.TRANSACTION_NAME";

    private ServletUtils() {
    }

    /**
     * If the exception is a <tt>javax.servlet.ServletException</tt> and the cause of the exception is not null, return
     * the cause. Otherwise, return the exception.
     * 
     * Since most uncaught exceptions are <tt>javax.servlet.ServletException</tt> it's more informative to report the
     * root cause to RPM.
     * 
     */
    public static Throwable getReportError(Throwable throwable) {
        if (throwable != null) {
            if (SERVLET_EXCEPTION_CLASS_NAME.equals(throwable.getClass().getName())) {
                return StackTraces.getRootCause(throwable);
            }
        }
        return throwable;
    }

    public static Map<String, String> getSimpleParameterMap(Map<String, String[]> parameterMap, int maxSizeLimit) {
        if (parameterMap == null || parameterMap.isEmpty()) {
            return Collections.emptyMap();
        }
        Map<String, String> parameters = new HashMap<String, String>();
        for (Entry<String, String[]> entry : parameterMap.entrySet()) {
            String name = entry.getKey();
            String[] values = entry.getValue();
            String value = getValue(values, maxSizeLimit);
            if (value != null) {
                parameters.put(name, value);
            }
        }
        return parameters;
    }

    public static void recordParameters(Transaction tx, Request request) {
        if (tx.isIgnore()) {
            return;
        }

        if (!ServiceFactory.getAttributesService().captureRequestParams(tx.getApplicationName())) {
            return;
        }
        Map<String, String> requestParameters = getRequestParameterMap(request,
                tx.getAgentConfig().getMaxUserParameterSize());
        if (requestParameters.isEmpty()) {
            return;
        }
        tx.getPrefixedAgentAttributes().put(AttributeNames.HTTP_REQUEST_PREFIX, requestParameters);
    }

    static Map<String, String> getRequestParameterMap(Request request, int maxSizeLimit) {
        Enumeration<?> nameEnumeration = request.getParameterNames();
        if (nameEnumeration == null || !nameEnumeration.hasMoreElements()) {
            return Collections.emptyMap();
        }
        Map<String, String> requestParameters = new HashMap<String, String>();

        while (nameEnumeration.hasMoreElements()) {
            String name = nameEnumeration.nextElement().toString();
            if (name.length() > maxSizeLimit) {
                Agent.LOG.log(Level.FINER,
                        "Rejecting request parameter with key \"{0}\" because the key is over the size limit of {1}",
                        name, maxSizeLimit);
            } else {
                String[] values = request.getParameterValues(name);
                String value = getValue(values, maxSizeLimit);
                if (value != null) {
                    requestParameters.put(name, value);
                }
            }
        }
        return requestParameters;
    }

    private static String getValue(String[] values, int maxSizeLimit) {
        if (values == null || values.length == 0) {
            return null;
        }
        String value = values.length == 1 ? values[0] : Arrays.asList(values).toString();
        if (value != null && value.length() > maxSizeLimit) {
            if (values.length == 1) {
                value = value.substring(0, maxSizeLimit);
            } else {
                value = value.substring(0, maxSizeLimit - 1) + ']';
            }
        }
        return value;
    }
}

==> newrelic-agent-8.7.0-sources.jar: com/newrelic/agent/servlet/ServletUtils.java <==
/*
 *
 *  * Copyright 2020 New Relic Corporation. All rights reserved.
 *  * SPDX-License-Identifier: Apache-2.0
 *
 */

package com.newrelic.agent.servlet;

import com.newrelic.agent.Agent;
import com.newrelic.agent.Transaction;
import com.newrelic.agent.attributes.AttributeNames;
import com.newrelic.agent.config.ConfigConstant;
import com.newrelic.agent.service.ServiceFactory;
import com.newrelic.api.agent.Request;

import java.util.Arrays;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;

public class ServletUtils {

    private ServletUtils() {
    }

    public static void recordParameters(Transaction tx, Request request) {
        if (tx.isIgnore()) {
            return;
        }

        if (!ServiceFactory.getAttributesService().captureRequestParams(tx.getApplicationName())) {
            return;
        }
        Map<String, String> requestParameters = getRequestParameterMap(request);
        if (requestParameters.isEmpty()) {
            return;
        }
        tx.getPrefixedAgentAttributes().put(AttributeNames.HTTP_REQUEST_PREFIX, requestParameters);
    }

    static Map<String, String> getRequestParameterMap(Request request) {
        Enumeration<?> nameEnumeration = request.getParameterNames();
        if (nameEnumeration == null || !nameEnumeration.hasMoreElements()) {
            return Collections.emptyMap();
        }
        Map<String, String> requestParameters = new HashMap<>();

        while (nameEnumeration.hasMoreElements()) {
            String name = nameEnumeration.nextElement().toString();
            if (name.length() > ConfigConstant.MAX_USER_ATTRIBUTE_SIZE) {
                Agent.LOG.log(Level.FINER,
                        "Rejecting request parameter with key \"{0}\" because the key is over the size limit of {1}",
                        name, ConfigConstant.MAX_USER_ATTRIBUTE_SIZE);
            } else {
                String[] values = request.getParameterValues(name);
                String value = getValue(values);
                if (value != null) {
                    requestParameters.put(name, value);
                }
            }
        }
        return requestParameters;
    }

    private static String getValue(String[] values) {
        if (values == null || values.length == 0) {
            return null;
        }
        String value = values.length == 1 ? values[0] : Arrays.asList(values).toString();
        if (value != null && value.length() > ConfigConstant.MAX_USER_ATTRIBUTE_SIZE) {
            if (values.length == 1) {
                value = value.substring(0, ConfigConstant.MAX_USER_ATTRIBUTE_SIZE);
            } else {
                value = value.substring(0, ConfigConstant.MAX_USER_ATTRIBUTE_SIZE - 1) + ']';
            }
        }
        return value;
    }
}
